<?php
/**
 * Swoosh_Storage Class
 *
 * Copyright 2003-2005 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: swoosh/lib/Storage.php,v 1.17 2005/01/03 14:35:32 jan Exp $
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.17 $
 * @package Swoosh
 */
class Swoosh_Storage {

    /**
     * A hash containing any parameters for the current storage driver.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Constructor
     *
     * @param optional array $params  Any parameters needed for this storage
     *                                driver.
     */
    function Swoosh_Storage($params)
    {
        $this->_params = $params;
    }

    /**
     * Fetches a list of messages submitted to the gateway and optionally
     * individual send part of each message.
     *
     * @public
     *
     * @param bool  True or false whether to include information about
     *              individual sends for each message.
     *
     * @return array  A list of messages submitted selection box.
     */
    function getMessages($with_sends = false)
    {
        $messages = $this->_getMessages();

        /* Are sends needed. */
        if ($with_sends && count($messages)) {
            $sends = $this->_getSends(array_keys($messages));
            /* Loop through all sends and include them in the messages
             * array. */
            foreach ($sends as $message_id => $send) {
                $messages[$message_id]['sends'] = $send;
            }
        }

        return $messages;
    }

    /**
     * Fetches a list of gateways to be used in a selection box, for example
     * when sending a message and the user can select which gateway to use.
     *
     * @return array  The list of gateways available that can be used in a
     *                selection box.
     */
    function getGatewaySelect()
    {
        /* Get the list of gateways from the backend. */
        $gateway_list = $this->getGateways();
        if (is_a($gateway_list, 'PEAR_Error')) {
            return array();
        }

        /* Rearrange the array to have gateway id as key to each
         * element. */
        $select_list = array();
        foreach ($gateway_list as $gateway) {
            $select_list[$gateway['gateway_id']] = $gateway['gateway_name'];
        }

        return $select_list;
    }

    /**
     * Attempts to return a concrete Storage instance based on $driver.
     *
     * @param string $driver          The type of concrete Storage subclass to
     *                                return. This is based on the storage
     *                                driver ($driver). The code is dynamically
     *                                included.
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return object Swoosh_Storage  The newly created concrete Storage
     *                                instance, or false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Storage/' . $driver . '.php';
        $class = 'Swoosh_Storage_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Swoosh_Storage instance
     * based on $driver. It wil only create a new instance if no Swoosh_Storage
     * instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Swoosh_Storage::singleton()
     *
     * @param string $driver          The type of concrete Swoosh_Storage
     *                                subclass to return. The is based on the
     *                                storage driver ($driver). The code is
     *                                dynamically included.
     *
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return mixed  The created concrete Swoosh_Storage instance, or false on
     *                error.
     */
    function &singleton($driver, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Swoosh_Storage::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
