<?php
/**
 * Swoosh_Storage_sql Class
 *
 * Copyright 2003-2005 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: swoosh/lib/Storage/sql.php,v 1.17 2005/01/03 14:35:33 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.17 $
 * @package Swoosh
 */
class Swoosh_Storage_sql extends Swoosh_Storage {

    var $_db;

    /**
     * Fetches a list of available gateways.
     *
     * @return array  An array of the available gateways.
     */
    function getGateways()
    {
        /* Get the gateways. */
        $sql = 'SELECT * FROM swoosh_gateways';
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::_getGateways(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        return $result;
    }

    /**
     * Fetches a gateway from the backend.
     *
     * @public
     *
     * @param int $gateway_id  The gateway id to fetch.
     *
     * @return array  An array containing the gateway settings and parameters.
     */
    function &getGateway($gateway_id)
    {
        /* Get the gateways. */
        $sql = sprintf('SELECT * FROM swoosh_gateways WHERE gateway_id = %s',
                       (int)$gateway_id);
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::getGateway(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $gateway = $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
        if (is_a($gateway, 'PEAR_Error')) {
            Horde::logMessage($gateway, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $gateway;
        }

        /* Unserialize the gateway params. */
        require_once 'Horde/Serialize.php';
        $gateway['gateway_params'] = Horde_Serialize::unserialize($gateway['gateway_params'], SERIALIZE_UTF7_BASIC);

        /* Unserialize the gateway send params. */
        $gateway['gateway_sendparams'] = Horde_Serialize::unserialize($gateway['gateway_sendparams'], SERIALIZE_UTF7_BASIC);

        return $gateway;
    }

    /**
     * Saves a gateway to the backend.
     *
     * @public
     *
     * @param array $info  The gateway settings to be saved passed by reference
     *                     as an array.
     *
     * @return mixed  Gateway id on success or a PEAR error on failure.
     */
    function saveGateway(&$info)
    {
        if (empty($info['gateway_id'])) {
            /* No existing gateway id, so new gateway and get next id. */
            $info['gateway_id'] = $this->_db->nextId('swoosh_gateways');
            if (is_a($info['gateway_id'], 'PEAR_Error')) {
                Horde::logMessage($info['gateway_id'], __FILE__, __LINE__, PEAR_LOG_ERR);
                return $info['gateway_id'];
            }
            $sql_sprintf = 'INSERT INTO swoosh_gateways (gateway_id, gateway_driver, gateway_name, gateway_params, gateway_sendparams) VALUES (%s, %s, %s, %s, %s)';
        } else {
            /* Existing gateway id, so editing an existing gateway. */
            $sql_sprintf = 'UPDATE swoosh_gateways SET gateway_id = %s, gateway_driver = %s, gateway_name = %s, gateway_params = %s, gateway_sendparams = %s' . sprintf(' WHERE gateway_id = %s', (int)$info['gateway_id']);
        }

        /* Serialize the gateway params. */
        require_once 'Horde/Serialize.php';
        if (!empty($info['gateway_params'])) {
            $info['gateway_params'] = $this->_db->quote(Horde_Serialize::serialize($info['gateway_params'], SERIALIZE_UTF7_BASIC));
        } else {
            $info['gateway_params'] = 'NULL';
        }

        /* Serialize the gateway send params. */
        if (!empty($info['gateway_sendparams'])) {
            $info['gateway_sendparams'] = $this->_db->quote(Horde_Serialize::serialize($info['gateway_sendparams'], SERIALIZE_UTF7_BASIC));
        } else {
            $info['gateway_sendparams'] = 'NULL';
        }

        /* Put together the sql statement. */
        $sql = sprintf($sql_sprintf,
                       (int)$info['gateway_id'],
                       $this->_db->quote($info['gateway_driver']),
                       $this->_db->quote($info['gateway_name']),
                       $info['gateway_params'],
                       $info['gateway_sendparams']);
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::saveGateway(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $info['gateway_id'];
    }

    /**
     * Deletes a gateway from the backend.
     *
     * @public
     *
     * @param int $gateway_id  The gateway id of the gateway to delete.
     *
     * @return mixed  True on success or a PEAR error on failure.
     */
    function deleteGateway($gateway_id)
    {
        $sql = sprintf('DELETE FROM swoosh_gateways WHERE gateway_id = %s',
                       (int)$gateway_id);
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::deleteGateway(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        return $result;
    }

    /**
     * Saves a message to the backend.
     *
     * @public
     *
     * @param int    $gateway_id                 The id of the gateway used to
     *                                           send this message.
     * @param string $message_text               The text of the message.
     * @param string $message_params             Any send params used for this
     *                                           message.
     * @param optional string $message_batch_id  If batch sending is used, the
     *                                           batch id of this message.
     *
     * @return mixed  True on success or PEAR Error on failure.
     */
    function saveMessage($gateway_id, $message_text, $message_params, $message_batch_id = null)
    {
        $message_id = $this->_db->nextId('swoosh_messages');
        if (is_a($message_id, 'PEAR_Error')) {
            Horde::logMessage($message_id, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $message_id;
        }

        /* Serialize the message params. */
        require_once 'Horde/Serialize.php';
        $message_params = Horde_Serialize::serialize($message_params, SERIALIZE_UTF7_BASIC);

        $sql = sprintf('INSERT INTO swoosh_messages (message_id, user_uid, gateway_id, message_batch_id, message_text, message_params, message_submitted) VALUES (%s, %s, %s, %s, %s, %s, %s)',
                       (int)$message_id,
                       $this->_db->quote(Auth::getAuth()),
                       (int)$gateway_id,
                       is_null($message_batch_id) ? 'NULL' : (int)$message_batch_id,
                       $this->_db->quote($message_text),
                       $this->_db->quote($message_params),
                       (int)time());
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $message_id;
    }

    /**
     * Saves an individual send to the backend. This will be one instance of
     * a message being sent to a recipient.
     *
     * @public
     *
     * @param int    $message_id  The message id.
     * @param string $remote_id   The text of the message.
     * @param string $recipient   Any send params used for this
     * @param string $error       Any send params used for this
     *
     * @return mixed  The send id on success or PEAR Error on failure.
     */
    function saveSend($message_id, $remote_id, $recipient, $error)
    {
        $send_id = $this->_db->nextId('swoosh_sends');
        if (is_a($send_id, 'PEAR_Error')) {
            Horde::logMessage($send_id, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $send_id;
        }
        $sql = sprintf('INSERT INTO swoosh_sends (send_id, message_id, send_remote_id, send_recipient, send_status) VALUES (%s, %s, %s, %s, %s)',
                       $send_id,
                       $message_id,
                       (is_null($remote_id) ? 'NULL' : $this->_db->quote($remote_id)),
                       $this->_db->quote($recipient),
                       (is_null($error) ? 'NULL' : $this->_db->quote($error)));
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $send_id;
    }

    /**
     * Gets all the messages for the current user.
     *
     * @private
     *
     * @return mixed  The array of messages on success or PEAR Error on failure.
     */
    function _getMessages()
    {
        $sql = sprintf('SELECT * FROM swoosh_messages WHERE user_uid = %s',
                       $this->_db->quote(Auth::getAuth()));
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::_getMessages(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAssoc($sql, false, null, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        return $result;
    }

    /**
     * Fetches all sends for one or more message ids.
     *
     * @private
     * @param int|array  $message_id  The message id(s).
     *
     * @return mixed  The send id on success or PEAR Error on failure.
     */
    function _getSends($message_ids)
    {
        if (!is_array($message_ids)) {
            $message_ids = array($message_ids);
        }

        $sql = sprintf('SELECT message_id, swoosh_sends.* FROM swoosh_sends WHERE message_id IN (%s)',
                       implode(', ', $message_ids));
        Horde::logMessage('SQL Query by Swoosh_Storage_sql::_getSends(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAssoc($sql, false, null, DB_FETCHMODE_ASSOC, true);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        return $result;
    }

    function initialise()
    {
        global $registry;

        Horde::assertDriverConfig($this->_params, 'sql',
            array('phptype', 'hostspec', 'username', 'database'));

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        return true;
    }

}
