<?php
/**
 * The Swoosh:: class providing some support functions to the Swoosh module.
 *
 * Copyright 2003-2005 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: swoosh/lib/Swoosh.php,v 1.19 2005/01/03 14:35:32 jan Exp $
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.19 $
 * @package Swoosh
 */
class Swoosh {

    /**
     * Returns the column to sort by, checking first if it is specified in the
     * URL, then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return string  The column to sort by.
     */
    function getSortBy($view)
    {
        global $prefs;
        if (($sortby = Util::getFormData($view . '_sortby')) !== null) {
            $prefs->setValue($view . '_sortby', $sortby);
        }
        return $prefs->getValue($view . '_sortby');
    }

    /**
     * Returns the sort direction, checking first if it is specified in the URL,
     * then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return int  The sort direction, 0 = ascending, 1 = descending.
     */
    function getSortDir($view)
    {
        global $prefs;
        if (($sortdir = Util::getFormData($view . '_sortdir')) !== null) {
            $prefs->setValue($view . '_sortdir', $sortdir);
        }
        return $prefs->getValue($view . '_sortdir');
    }

    /**
     * Given a string, tries to expand any names by looking them up in the
     * turba addressbook.
     *
     * @access public
     *
     * @param string $destination  A string containing a mix of names and
     *                             numbers which will be parsed for any
     *                             possible expansion.
     *
     * @return array  An array potentially containing three elements: expanded,
     *                ambiguous and unmatched. No errors are reported as the
     *                script calling this should handle any error reporting.
     */
    function expandNumbers($destination, $fixed_ambiguous = null)
    {
        if (!preg_match('|[^\s]|', $destination)) {
            return '';
        }

        global $prefs, $registry;

        $src = explode("\t", $prefs->getValue('search_sources'));
        if ((count($src) == 1) && empty($src[0])) {
            $src = array();
        }

        $search_fields = array();
        if (($val = $prefs->getValue('search_fields'))) {
            $field_arr = explode("\n", $val);
            foreach ($field_arr as $field) {
                $field = trim($field);
                if (!empty($field)) {
                    $tmp = explode("\t", $field);
                    if (count($tmp) > 1) {
                        $source = array_splice($tmp, 0, 1);
                        $search_fields[$source[0]] = $tmp;
                    }
                }
            }
        }

        $matched = array();
        $search_list = array();
        $duplicates = array();

        /* Explode the destinations. */
        $destination = preg_replace('/(,\s+|\n|(\r\n))/', ',', $destination);
        preg_match_all('/(((".*?,?.*?"))?.*?)(,|$)/', $destination, $matches, PREG_SET_ORDER);
        $i = 0;
        foreach ($matches as $m) {
            /* Merge any ambiguous destinations that have been fixed into
             * the expanded list. */
            while (isset($fixed_ambiguous[$i])) {
                $matched[$i] = $fixed_ambiguous[$i++];
                unset($fixed_ambiguous[$i]);
            }
            $m[1] = trim($m[1]);
            if (empty($m[1])) {
                continue;
            } elseif (preg_match('/^(.*?<)?(\+?\d{7,})(>|$)/', $m[1])) {
                /* The above checks if it is a vaguely valid number enclosed in
                 * <> brackets or just a number on its own. The final number
                 * validation will only have to be done by the gateway API as
                 * each one has their own format. If this does look like
                 * already expanded or a valid number, skip it. */
                $matched[$i++] = $m[1];
                continue;
            } elseif (isset($search_list[$m[1]])) {
                /* Duplicate, don't bother adding it to the list. */
                $matched[$i++] = $m[1];
                continue;
            }
            $search_list[$m[1]] = $i++;
        }
        $search_list = array_flip($search_list);

        $results = $registry->call('contacts/search', array($search_list, $src, $search_fields));
        if (is_a($results, 'PEAR_Error')) {
            return $results;
        }

        $ambiguous = array();
        $unmatched = array();

        foreach ($search_list as $i => $name) {
            if (!empty($results[$name])) {
                if (count($results[$name]) == 1) {
                    /* Matched an entry. */
                    $matched[$i] = $results[$name][0]['name'] . ' <' . $results[$name][0]['cellPhone'] . '>';
                } else {
                    /* Handle the multiple case - we return an array with all
                     * found addresses. */
                    foreach ($results[$name] as $res) {
                        $ambiguous[$name . ':' . $i][] = $res['name'] . ' <' . $res['cellPhone'] . '>';
                    }
                }
            } else {
                /* Handle the missing/invalid case - insert into the unmatched
                 * array. */
                $unmatched[$i] = $search_list[$i];
            }
        }

        $expanded = array('matched' => array());
        /* Sort and insert any matched destinations. */
        if (!empty($matched)) {
            ksort($matched);
            $expanded = array('matched' => $matched);
        }
        /* Merge in at the end any remaining ambiguous fixed destinations. */
        if (!empty($fixed_ambiguous)) {
            $expanded['matched'] += $fixed_ambiguous;
        }
        /* Sort and insert any ambiguous destinations which will require
         * further clarification from the user. */
        if (!empty($ambiguous)) {
            ksort($ambiguous);
            $expanded['ambiguous'] = $ambiguous;
        }
        /* Sort and insert any unmatched destinations. */
        if (!empty($unmatched)) {
            ksort($unmatched);
            $expanded['unmatched'] = $unmatched;
        }
        return $expanded;
    }

    function getMessageStatus($sends)
    {
    }

    /**
     * Build Swoosh's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        /* Send message. */
        $menu->addArray(array('url' => Horde::applicationUrl('send.php'), 'text' => _("Send"), 'icon' => 'swoosh.png'));

        /* List sent messages. */
        $menu->addArray(array('url' => Horde::applicationUrl('messages/index.php'), 'text' => _("Messages"), 'icon' => 'messages.png'));

        /* Gateway admin. */
        if (Auth::isAdmin()) {
            $menu->addArray(array('url' => Horde::applicationUrl('gateways/index.php'), 'text' => _("Gateways"), 'icon' => 'gateways.png'));
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
